@@ -6,31 +6,16 @@ from PIL import ImageChops
 
 import histolab.filters.image_filters_functional as imf
 
-from ..fixtures import GS, RGB, RGBA
+from ..fixtures import GS, NPY, RGB, RGBA
 from ..util import load_expectation
 
 
-@pytest.mark.parametrize(
-    "pil_image, expected_image",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "pil-images-rgb/diagnostic-slide-thumb-rgb-inverted",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "pil-images-rgb/diagnostic-slide-thumb-hsv-inverted",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "pil-images-rgb/diagnostic-slide-thumb-ycbcr-inverted",
-        ),
-    ),
-)
-def test_invert_filter_with_rgb_image(pil_image, expected_image):
-    expected_value = load_expectation(expected_image, type_="png")
+def test_invert_filter_with_rgb_image():
+    expected_value = load_expectation(
+        "pil-images-rgb/diagnostic-slide-thumb-rgb-inverted", type_="png"
+    )
 
-    inverted_img = imf.invert(pil_image)
+    inverted_img = imf.invert(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB)
 
     np.testing.assert_array_almost_equal(
         np.array(inverted_img), np.array(expected_value)
@@ -72,36 +57,20 @@ def test_invert_filter_with_gs_image():
     )
 
 
-@pytest.mark.parametrize(
-    "pil_image, expected_image",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "pil-images-rgb/diagnostic-slide-thumb-rgb-to-hed",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "pil-images-rgb/diagnostic-slide-thumb-hsv-rgb-to-hed",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "pil-images-rgb/diagnostic-slide-thumb-ycbcr-rgb-to-hed",
-        ),
-    ),
-)
-def test_rgb_to_hed_filter_with_rgb_image(pil_image, expected_image):
-    expected_value = load_expectation(expected_image, type_="png")
+def test_rgb_to_hed_filter_with_rgb_image():
+    expected_value = load_expectation(
+        "arrays/diagnostic-slide-thumb-rgb-to-hed", type_="npy"
+    )
 
-    hed_img = imf.rgb_to_hed(pil_image)
+    hed_arr = imf.rgb_to_hed(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB)
 
-    np.testing.assert_array_almost_equal(np.array(hed_img), np.array(expected_value))
-    assert np.unique(np.array(ImageChops.difference(hed_img, expected_value)))[0] == 0
+    np.testing.assert_array_almost_equal(hed_arr, expected_value)
 
 
 def test_rgb_to_hed_filter_with_rgba_image():
     img = RGBA.DIAGNOSTIC_SLIDE_THUMB
     expected_value = load_expectation(
-        "pil-images-rgba/diagnostic-slide-thumb-rgba-to-hed", type_="png"
+        "arrays/diagnostic-slide-thumb-rgb-to-hed", type_="npy"
     )
     expected_warning_regex = (
         r"Input image must be RGB. NOTE: the image will be converted to RGB before"
@@ -109,10 +78,9 @@ def test_rgb_to_hed_filter_with_rgba_image():
     )
 
     with pytest.warns(UserWarning, match=expected_warning_regex):
-        hed_img = imf.rgb_to_hed(img)
+        hed_arr = imf.rgb_to_hed(img)
 
-    np.testing.assert_array_almost_equal(np.array(hed_img), np.array(expected_value))
-    assert np.unique(np.array(ImageChops.difference(hed_img, expected_value)))[0] == 0
+    np.testing.assert_array_almost_equal(hed_arr, expected_value)
 
 
 def test_rgb_to_hed_raises_exception_on_gs_image():
@@ -208,26 +176,14 @@ def test_eosin_channel_raises_exception_on_gs_image():
     assert str(err.value) == "Input image must be RGB/RGBA."
 
 
-@pytest.mark.parametrize(
-    "pil_image, expected_image",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "pil-images-rgb/diagnostic-slide-thumb-rgb-to-hsv",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "pil-images-rgb/diagnostic-slide-thumb-ycbcr-rgb-to-hsv",
-        ),
-    ),
-)
-def test_rgb_to_hsv_filter_with_rgb_image(pil_image, expected_image):
-    expected_value = load_expectation(expected_image, type_="png")
+def test_rgb_to_hsv_filter_with_rgb_image():
+    expected_value = load_expectation(
+        "arrays/diagnostic-slide-thumb-rgb-to-hsv", type_="npy"
+    )
 
-    hsv_img = imf.rgb_to_hsv(pil_image)
+    hsv_arr = imf.rgb_to_hsv(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB)
 
-    np.testing.assert_array_almost_equal(np.array(hsv_img), np.array(expected_value))
-    assert np.unique(np.array(ImageChops.difference(hsv_img, expected_value)))[0] == 0
+    np.testing.assert_array_almost_equal(hsv_arr, expected_value)
 
 
 def test_rgb_to_hsv_raises_exception_on_rgba_image():
@@ -267,27 +223,12 @@ def test_stretch_contrast_filter_on_rgba_image():
     )
 
 
-@pytest.mark.parametrize(
-    "pil_image, expected_image",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "pil-images-rgb/diagnostic-slide-thumb-rgb-stretch-contrast",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "pil-images-rgb/diagnostic-slide-thumb-ycbcr-stretch-contrast",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "pil-images-rgb/diagnostic-slide-thumb-hsv-stretch-contrast",
-        ),
-    ),
-)
-def test_stretch_contrast_filter_on_rgb_image(pil_image, expected_image):
-    expected_value = load_expectation(expected_image, type_="png")
+def test_stretch_contrast_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "pil-images-rgb/diagnostic-slide-thumb-rgb-stretch-contrast", type_="png"
+    )
 
-    stretched_img = imf.stretch_contrast(pil_image)
+    stretched_img = imf.stretch_contrast(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB)
 
     np.testing.assert_array_almost_equal(
         np.array(stretched_img), np.array(expected_value)
@@ -355,27 +296,12 @@ def test_histogram_equalization_filter_on_rgba_image():
     )
 
 
-@pytest.mark.parametrize(
-    "pil_image, expected_image",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "pil-images-rgb/diagnostic-slide-thumb-rgb-histogram-equalization",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "pil-images-rgb/diagnostic-slide-thumb-ycbcr-histogram-equalization",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "pil-images-rgb/diagnostic-slide-thumb-hsv-histogram-equalization",
-        ),
-    ),
-)
-def test_histogram_equalization_filter_on_rgb_image(pil_image, expected_image):
-    expected_value = load_expectation(expected_image, type_="png")
+def test_histogram_equalization_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "pil-images-rgb/diagnostic-slide-thumb-rgb-histogram-equalization", type_="png"
+    )
 
-    hist_equ_img = imf.histogram_equalization(pil_image, 200)
+    hist_equ_img = imf.histogram_equalization(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB, 200)
 
     np.testing.assert_array_almost_equal(
         np.array(hist_equ_img), np.array(expected_value)
@@ -417,27 +343,12 @@ def test_adaptive_equalization_filter_on_rgba_image():
     )
 
 
-@pytest.mark.parametrize(
-    "pil_image, expected_image",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "pil-images-rgb/diagnostic-slide-thumb-rgb-adaptive-equalization",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "pil-images-rgb/diagnostic-slide-thumb-ycbcr-adaptive-equalization",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "pil-images-rgb/diagnostic-slide-thumb-hsv-adaptive-equalization",
-        ),
-    ),
-)
-def test_adaptive_equalization_filter_on_rgb_image(pil_image, expected_image):
-    expected_value = load_expectation(expected_image, type_="png")
+def test_adaptive_equalization_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "pil-images-rgb/diagnostic-slide-thumb-rgb-adaptive-equalization", type_="png"
+    )
 
-    adap_equ_img = imf.adaptive_equalization(pil_image, 200, 0.2)
+    adap_equ_img = imf.adaptive_equalization(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB, 200, 0.2)
 
     np.testing.assert_array_almost_equal(
         np.array(adap_equ_img), np.array(expected_value)
@@ -497,8 +408,6 @@ def test_local_equalization_filter_on_gs_image():
     "pil_rgb_image",
     (
         RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-        RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-        RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
         RGBA.DIAGNOSTIC_SLIDE_THUMB,
     ),
 )
@@ -510,46 +419,24 @@ def test_local_equalization_raises_exception_on_rgb_images(pil_rgb_image):
     assert str(err.value) == "Input must be 2D."
 
 
-def test_kmeans_segmentation_filter_on_rgba_image():
+def test_kmeans_segmentation_raises_value_error_on_rgba_images():
     rgba_img = RGBA.DIAGNOSTIC_SLIDE_THUMB
+
+    with pytest.raises(ValueError) as err:
+        imf.kmeans_segmentation(rgba_img)
+
+    assert isinstance(err.value, ValueError)
+    assert str(err.value) == "Input image cannot be RGBA"
+
+
+def test_kmeans_segmentation_filter_on_rgb_image():
     expected_value = load_expectation(
-        "pil-images-rgba/diagnostic-slide-thumb-kmeans-segmentation", type_="png"
+        "pil-images-rgb/diagnostic-slide-thumb-rgb-kmeans-segmentation", type_="png"
     )
 
-    kmeans_segmentation_img = imf.kmeans_segmentation(rgba_img, 800, 10)
-
-    np.testing.assert_array_almost_equal(
-        np.array(kmeans_segmentation_img), np.array(expected_value)
+    kmeans_segmentation_img = imf.kmeans_segmentation(
+        RGB.DIAGNOSTIC_SLIDE_THUMB_RGB, 800, 10
     )
-    assert (
-        np.unique(
-            np.array(ImageChops.difference(kmeans_segmentation_img, expected_value))
-        )[0]
-        == 0
-    )
-
-
-@pytest.mark.parametrize(
-    "pil_image, expected_image",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "pil-images-rgb/diagnostic-slide-thumb-rgb-kmeans-segmentation",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "pil-images-rgb/diagnostic-slide-thumb-ycbcr-kmeans-segmentation",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "pil-images-rgb/diagnostic-slide-thumb-hsv-kmeans-segmentation",
-        ),
-    ),
-)
-def test_kmeans_segmentation_filter_on_rgb_image(pil_image, expected_image):
-    expected_value = load_expectation(expected_image, type_="png")
-
-    kmeans_segmentation_img = imf.kmeans_segmentation(pil_image, 800, 10)
 
     np.testing.assert_array_almost_equal(
         np.array(kmeans_segmentation_img), np.array(expected_value)
@@ -565,7 +452,7 @@ def test_kmeans_segmentation_filter_on_rgb_image(pil_image, expected_image):
 def test_kmeans_segmentation_filter_on_gs_image():
     gs_img = GS.DIAGNOSTIC_SLIDE_THUMB_GS
     expected_value = load_expectation(
-        "pil-images-gs/diagnostic-slide-thumb-gs-kmeans-segmentation", type_="png"
+        "pil-images-rgb/diagnostic-slide-thumb-gs-kmeans-segmentation", type_="png"
     )
 
     kmeans_segmentation_img = imf.kmeans_segmentation(gs_img, 800, 10)
@@ -581,27 +468,12 @@ def test_kmeans_segmentation_filter_on_gs_image():
     )
 
 
-@pytest.mark.parametrize(
-    "pil_image, expected_image",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "pil-images-rgb/diagnostic-slide-thumb-rgb-rag-threshold",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "pil-images-rgb/diagnostic-slide-thumb-ycbcr-rag-threshold",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "pil-images-rgb/diagnostic-slide-thumb-hsv-rag-threshold",
-        ),
-    ),
-)
-def test_rag_threshold_filter_on_rgb_image(pil_image, expected_image):
-    expected_value = load_expectation(expected_image, type_="png")
+def test_rag_threshold_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "pil-images-rgb/diagnostic-slide-thumb-rgb-rag-threshold", type_="png"
+    )
 
-    rag_threshold_img = imf.rag_threshold(pil_image, 650, 20.6, 9)
+    rag_threshold_img = imf.rag_threshold(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB, 650, 20.6, 9)
 
     np.testing.assert_array_almost_equal(
         np.array(rag_threshold_img), np.array(expected_value)
@@ -615,7 +487,7 @@ def test_rag_threshold_filter_on_rgb_image(pil_image, expected_image):
 def test_rag_threshold_filter_on_gs_image():
     gs_img = GS.DIAGNOSTIC_SLIDE_THUMB_GS
     expected_value = load_expectation(
-        "pil-images-gs/diagnostic-slide-thumb-gs-rag-threshold", type_="png"
+        "pil-images-rgb/diagnostic-slide-thumb-gs-rag-threshold", type_="png"
     )
 
     rag_threshold_img = imf.rag_threshold(gs_img, 650, 20.6, 15)
@@ -658,27 +530,14 @@ def test_hysteresis_threshold_filter_on_rgba_image():
     )
 
 
-@pytest.mark.parametrize(
-    "pil_image, expected_image",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "pil-images-rgb/diagnostic-slide-thumb-rgb-hysteresis-threshold",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "pil-images-rgb/diagnostic-slide-thumb-ycbcr-hysteresis-threshold",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "pil-images-rgb/diagnostic-slide-thumb-hsv-hysteresis-threshold",
-        ),
-    ),
-)
-def test_hysteresis_threshold_filter_on_rgb_image(pil_image, expected_image):
-    expected_value = load_expectation(expected_image, type_="png")
+def test_hysteresis_threshold_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "pil-images-rgb/diagnostic-slide-thumb-rgb-hysteresis-threshold", type_="png"
+    )
 
-    hysteresis_threshold_img = imf.hysteresis_threshold(pil_image, 10.6, 200)
+    hysteresis_threshold_img = imf.hysteresis_threshold(
+        RGB.DIAGNOSTIC_SLIDE_THUMB_RGB, 10.6, 200
+    )
 
     np.testing.assert_array_almost_equal(
         np.array(hysteresis_threshold_img), np.array(expected_value)
@@ -766,8 +625,6 @@ def test_local_otsu_threshold_filter_on_gs_image(pil_image, expected_image, disk
         (RGBA.DIAGNOSTIC_SLIDE_THUMB, 6, ValueError, "Input must be 2D."),
         (RGBA.DIAGNOSTIC_SLIDE_THUMB, -10, ValueError, "Input must be 2D."),
         (RGB.DIAGNOSTIC_SLIDE_THUMB_RGB, 10, ValueError, "Input must be 2D."),
-        (RGB.DIAGNOSTIC_SLIDE_THUMB_HSV, 0, ValueError, "Input must be 2D."),
-        (RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR, -3, ValueError, "Input must be 2D."),
         (
             GS.DIAGNOSTIC_SLIDE_THUMB_GS,
             -10,
@@ -838,18 +695,6 @@ def test_hysteresis_threshold_mask_raises_exception_on_thresholds(low, high):
             -20,
             -20,
         ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "mask-arrays/diagnostic-slide-thumb-ycbcr-hysteresis-threshold-mask",
-            -20,
-            100,
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "mask-arrays/diagnostic-slide-thumb-hsv-hysteresis-threshold-mask",
-            30,
-            -90,
-        ),
     ),
 )
 def test_hysteresis_threshold_mask_filter_on_rgb_image(
@@ -896,32 +741,19 @@ def test_otsu_threshold_filter_on_rgba_image():
     np.testing.assert_array_equal(otsu_threshold_mask, expected_value)
 
 
-@pytest.mark.parametrize(
-    "pil_image, expected_array",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "mask-arrays/diagnostic-slide-thumb-gs-otsu-threshold-mask",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "mask-arrays/diagnostic-slide-thumb-ycbcr-otsu-threshold-mask",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "mask-arrays/diagnostic-slide-thumb-hsv-otsu-threshold-mask",
-        ),
-    ),
-)
-def test_otsu_threshold_filter_on_rgb_image(pil_image, expected_array):
-    expected_value = load_expectation(expected_array, type_="npy")
+def test_otsu_threshold_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "mask-arrays/diagnostic-slide-thumb-gs-otsu-threshold-mask", type_="npy"
+    )
     expected_warning_regex = (
         r"otsu_threshold is expected to work correctly only for grayscale images.NOTE: "
         r"the image will be converted to grayscale before applying Otsuthreshold"
     )
 
     with pytest.warns(UserWarning, match=expected_warning_regex):
-        otsu_threshold_mask = imf.otsu_threshold(pil_image, operator.gt)
+        otsu_threshold_mask = imf.otsu_threshold(
+            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB, operator.gt
+        )
 
     np.testing.assert_array_equal(otsu_threshold_mask, expected_value)
 
@@ -953,11 +785,9 @@ def test_filter_entropy_filter_on_gs_image():
     (
         RGBA.DIAGNOSTIC_SLIDE_THUMB,
         RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-        RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-        RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
     ),
 )
-def test_filter_entropy_raises_exception_on_rgb_image(pil_image):
+def test_filter_entropy_raises_exception_on_rgb_or_rgba_image(pil_image):
     with pytest.raises(Exception) as err:
         imf.filter_entropy(pil_image)
 
@@ -981,11 +811,9 @@ def test_canny_edges_filter_on_gs_image():
     (
         RGBA.DIAGNOSTIC_SLIDE_THUMB,
         RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-        RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-        RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
     ),
 )
-def test_canny_edges_raises_exception_on_rgb_image(pil_image):
+def test_canny_edges_raises_exception_on_rgb_or_rgba_image(pil_image):
     with pytest.raises(Exception) as err:
         imf.canny_edges(pil_image)
 
@@ -1004,30 +832,12 @@ def test_grays_filter_on_rgba_image():
     np.testing.assert_array_equal(grays_mask, expected_value)
 
 
-@pytest.mark.parametrize(
-    "pil_image, expected_value, threshold",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "mask-arrays/diagnostic-slide-thumb-rgb-grays-mask",
-            20,
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "mask-arrays/diagnostic-slide-thumb-hsv-grays-mask",
-            0,
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "mask-arrays/diagnostic-slide-thumb-ycbcr-grays-mask",
-            -5,
-        ),
-    ),
-)
-def test_grays_filter_on_rgb_image(pil_image, expected_value, threshold):
-    expected_value = load_expectation(expected_value, type_="npy")
+def test_grays_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "mask-arrays/diagnostic-slide-thumb-rgb-grays-mask", type_="npy"
+    )
 
-    grays_mask = imf.grays(pil_image, threshold)
+    grays_mask = imf.grays(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB, 20)
 
     np.testing.assert_array_equal(grays_mask, expected_value)
 
@@ -1064,38 +874,12 @@ def test_red_filter_on_rgba_image(
     np.testing.assert_array_equal(red_filter_mask, expected_value)
 
 
-@pytest.mark.parametrize(
-    "pil_img, red_thresh, green_thresh, blue_thresh, expected_value",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            15,
-            200,
-            50,
-            "mask-arrays/diagnostic-slide-thumb-rgb-red-filter-mask",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            0,
-            20,
-            230,
-            "mask-arrays/diagnostic-slide-thumb-hsv-red-filter-mask",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            140,
-            160,
-            0,
-            "mask-arrays/diagnostic-slide-thumb-ycbcr-red-filter-mask",
-        ),
-    ),
-)
-def test_red_filter_on_rgb_image(
-    pil_img, red_thresh, green_thresh, blue_thresh, expected_value
-):
-    expected_value = load_expectation(expected_value, type_="npy")
+def test_red_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "mask-arrays/diagnostic-slide-thumb-rgb-red-filter-mask", type_="npy"
+    )
 
-    red_filter_mask = imf.red_filter(pil_img, red_thresh, green_thresh, blue_thresh)
+    red_filter_mask = imf.red_filter(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB, 15, 200, 50)
 
     np.testing.assert_array_equal(red_filter_mask, expected_value)
 
@@ -1130,38 +914,6 @@ def test_red_filter_on_rgb_image(
             ValueError,
             "RGB Thresholds must be in range [0, 255]",
         ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            0,
-            200,
-            500,
-            ValueError,
-            "RGB Thresholds must be in range [0, 255]",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            0,
-            200,
-            -500,
-            ValueError,
-            "RGB Thresholds must be in range [0, 255]",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            -20,
-            -40,
-            -90,
-            ValueError,
-            "RGB Thresholds must be in range [0, 255]",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            290,
-            30,
-            70,
-            ValueError,
-            "RGB Thresholds must be in range [0, 255]",
-        ),
     ),
 )
 def test_red_filter_raises_right_exceptions(
@@ -1185,27 +937,12 @@ def test_red_pen_filter_on_rgba_image():
     np.testing.assert_array_equal(red_pen_filter_img, expected_value)
 
 
-@pytest.mark.parametrize(
-    "pil_img, expected_value",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "pil-images-rgb/diagnostic-slide-thumb-rgb-red-pen-filter",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "pil-images-rgb/diagnostic-slide-thumb-hsv-red-pen-filter",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "pil-images-rgb/diagnostic-slide-thumb-ycbcr-red-pen-filter",
-        ),
-    ),
-)
-def test_red_pen_filter_on_rgb_image(pil_img, expected_value):
-    expected_value = load_expectation(expected_value, type_="png")
+def test_red_pen_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "pil-images-rgb/diagnostic-slide-thumb-rgb-red-pen-filter", type_="png"
+    )
 
-    red_pen_filter_img = imf.red_pen_filter(pil_img)
+    red_pen_filter_img = imf.red_pen_filter(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB)
 
     np.testing.assert_array_almost_equal(
         np.array(red_pen_filter_img), np.array(expected_value)
@@ -1282,38 +1019,12 @@ def test_green_channel_filter_with_wrong_threshold(green_threshold):
     assert str(err.value) == "threshold must be in range [0, 255]"
 
 
-@pytest.mark.parametrize(
-    "pil_img, red_thresh, green_thresh, blue_thresh, expected_value",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            15,
-            200,
-            50,
-            "mask-arrays/diagnostic-slide-thumb-rgb-green-filter-mask",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            0,
-            20,
-            230,
-            "mask-arrays/diagnostic-slide-thumb-hsv-green-filter-mask",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            140,
-            160,
-            0,
-            "mask-arrays/diagnostic-slide-thumb-ycbcr-green-filter-mask",
-        ),
-    ),
-)
-def test_green_filter_on_rgb_image(
-    pil_img, red_thresh, green_thresh, blue_thresh, expected_value
-):
-    expected_value = load_expectation(expected_value, type_="npy")
+def test_green_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "mask-arrays/diagnostic-slide-thumb-rgb-green-filter-mask", type_="npy"
+    )
 
-    red_filter_mask = imf.green_filter(pil_img, red_thresh, green_thresh, blue_thresh)
+    red_filter_mask = imf.green_filter(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB, 15, 200, 50)
 
     np.testing.assert_array_equal(red_filter_mask, expected_value)
 
@@ -1348,38 +1059,6 @@ def test_green_filter_on_rgb_image(
             ValueError,
             "RGB Thresholds must be in range [0, 255]",
         ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            0,
-            200,
-            500,
-            ValueError,
-            "RGB Thresholds must be in range [0, 255]",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            0,
-            200,
-            -500,
-            ValueError,
-            "RGB Thresholds must be in range [0, 255]",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            -20,
-            -40,
-            -90,
-            ValueError,
-            "RGB Thresholds must be in range [0, 255]",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            290,
-            30,
-            70,
-            ValueError,
-            "RGB Thresholds must be in range [0, 255]",
-        ),
     ),
 )
 def test_green_filter_raises_right_exceptions(
@@ -1403,27 +1082,12 @@ def test_green_pen_filter_on_rgba_image():
     np.testing.assert_array_equal(green_pen_filter_img, expected_value)
 
 
-@pytest.mark.parametrize(
-    "pil_img, expected_value",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "pil-images-rgb/diagnostic-slide-thumb-rgb-green-pen-filter",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "pil-images-rgb/diagnostic-slide-thumb-hsv-green-pen-filter",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "pil-images-rgb/diagnostic-slide-thumb-ycbcr-green-pen-filter",
-        ),
-    ),
-)
-def test_green_pen_filter_on_rgb_image(pil_img, expected_value):
-    expected_value = load_expectation(expected_value, type_="png")
+def test_green_pen_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "pil-images-rgb/diagnostic-slide-thumb-rgb-green-pen-filter", type_="png"
+    )
 
-    green_pen_filter_img = imf.green_pen_filter(pil_img)
+    green_pen_filter_img = imf.green_pen_filter(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB)
 
     np.testing.assert_array_equal(green_pen_filter_img, expected_value)
 
@@ -1447,38 +1111,12 @@ def test_blue_filter_on_rgba_image(
     np.testing.assert_array_equal(blue_filter_mask, expected_value)
 
 
-@pytest.mark.parametrize(
-    "pil_img, red_thresh, green_thresh, blue_thresh, expected_value",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            15,
-            200,
-            50,
-            "mask-arrays/diagnostic-slide-thumb-rgb-blue-filter-mask",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            0,
-            20,
-            230,
-            "mask-arrays/diagnostic-slide-thumb-hsv-blue-filter-mask",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            140,
-            160,
-            230,
-            "mask-arrays/diagnostic-slide-thumb-ycbcr-blue-filter-mask",
-        ),
-    ),
-)
-def test_blue_filter_on_rgb_image(
-    pil_img, red_thresh, green_thresh, blue_thresh, expected_value
-):
-    expected_value = load_expectation(expected_value, type_="npy")
+def test_blue_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "mask-arrays/diagnostic-slide-thumb-rgb-blue-filter-mask", type_="npy"
+    )
 
-    blue_filter_mask = imf.blue_filter(pil_img, red_thresh, green_thresh, blue_thresh)
+    blue_filter_mask = imf.blue_filter(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB, 15, 200, 50)
 
     np.testing.assert_array_equal(blue_filter_mask, expected_value)
 
@@ -1513,38 +1151,6 @@ def test_blue_filter_on_rgb_image(
             ValueError,
             "RGB Thresholds must be in range [0, 255]",
         ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            0,
-            200,
-            500,
-            ValueError,
-            "RGB Thresholds must be in range [0, 255]",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            0,
-            200,
-            -500,
-            ValueError,
-            "RGB Thresholds must be in range [0, 255]",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            -20,
-            -40,
-            -90,
-            ValueError,
-            "RGB Thresholds must be in range [0, 255]",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            290,
-            30,
-            70,
-            ValueError,
-            "RGB Thresholds must be in range [0, 255]",
-        ),
     ),
 )
 def test_blue_filter_raises_right_exceptions(
@@ -1568,27 +1174,12 @@ def test_blue_pen_filter_on_rgba_image():
     np.testing.assert_array_equal(blue_pen_filter_img, expected_value)
 
 
-@pytest.mark.parametrize(
-    "pil_img, expected_value",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "pil-images-rgb/diagnostic-slide-thumb-rgb-blue-pen-filter",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "pil-images-rgb/diagnostic-slide-thumb-hsv-blue-pen-filter",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "pil-images-rgb/diagnostic-slide-thumb-ycbcr-blue-pen-filter",
-        ),
-    ),
-)
-def test_blue_pen_filter_on_rgb_image(pil_img, expected_value):
-    expected_value = load_expectation(expected_value, type_="png")
+def test_blue_pen_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "pil-images-rgb/diagnostic-slide-thumb-rgb-blue-pen-filter", type_="png"
+    )
 
-    blue_pen_filter_img = imf.blue_pen_filter(pil_img)
+    blue_pen_filter_img = imf.blue_pen_filter(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB)
 
     np.testing.assert_array_equal(blue_pen_filter_img, expected_value)
 
@@ -1607,27 +1198,12 @@ def test_yen_threshold_filter_on_rgba_image():
     np.testing.assert_array_equal(yen_threshold_mask, expected_value)
 
 
-@pytest.mark.parametrize(
-    "pil_image, expected_array",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "mask-arrays/diagnostic-slide-thumb-rgb-yen-threshold-mask",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "mask-arrays/diagnostic-slide-thumb-ycbcr-yen-threshold-mask",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "mask-arrays/diagnostic-slide-thumb-hsv-yen-threshold-mask",
-        ),
-    ),
-)
-def test_yen_threshold_filter_on_rgb_image(pil_image, expected_array):
-    expected_value = load_expectation(expected_array, type_="npy")
+def test_yen_threshold_filter_on_rgb_image():
+    expected_value = load_expectation(
+        "mask-arrays/diagnostic-slide-thumb-rgb-yen-threshold-mask", type_="npy"
+    )
 
-    yen_threshold_mask = imf.yen_threshold(pil_image)
+    yen_threshold_mask = imf.yen_threshold(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB)
 
     np.testing.assert_array_equal(yen_threshold_mask, expected_value)
 
@@ -1643,30 +1219,14 @@ def test_yen_threshold_filter_on_gs_image():
     np.testing.assert_array_equal(yen_threshold_mask, expected_value)
 
 
-@pytest.mark.parametrize(
-    "pil_image, expected_image",
-    (
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
-            "pil-images-rgb/diagnostic-slide-thumb-rgb-to-lab",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_HSV,
-            "pil-images-rgb/diagnostic-slide-thumb-hsv-rgb-to-lab",
-        ),
-        (
-            RGB.DIAGNOSTIC_SLIDE_THUMB_YCBCR,
-            "pil-images-rgb/diagnostic-slide-thumb-ycbcr-rgb-to-lab",
-        ),
-    ),
-)
-def test_rgb_to_lab_filter_with_rgb_image(pil_image, expected_image):
-    expected_value = load_expectation(expected_image, type_="png")
+def test_rgb_to_lab_filter_with_rgb_image():
+    expected_value = load_expectation(
+        "arrays/diagnostic-slide-thumb-rgb-to-lab", type_="npy"
+    )
 
-    lab_img = imf.rgb_to_lab(pil_image)
+    lab_img = imf.rgb_to_lab(RGB.DIAGNOSTIC_SLIDE_THUMB_RGB)
 
-    np.testing.assert_array_almost_equal(np.array(lab_img), np.array(expected_value))
-    assert np.unique(np.array(ImageChops.difference(lab_img, expected_value)))[0] == 0
+    np.testing.assert_array_almost_equal(lab_img, expected_value)
 
 
 @pytest.mark.parametrize(
@@ -1718,3 +1278,65 @@ def test_dab_channel_raises_exception_on_gs_image():
 
     assert isinstance(err.value, Exception)
     assert str(err.value) == "Input image must be RGB/RGBA."
+
+
+def test_rgb_to_od_filter_with_rgb_image():
+    expected_value = load_expectation(
+        "arrays/diagnostic-slide-thumb-rgb-to-od", type_="npy"
+    )
+
+    od_img = imf.rgb_to_od(
+        RGB.DIAGNOSTIC_SLIDE_THUMB_RGB,
+    )
+
+    np.testing.assert_allclose(od_img, expected_value)
+
+
+def test_rgb_to_od_filter_with_rgba_image():
+    img = RGBA.TCGA_LUNG
+    expected_value = load_expectation("arrays/tcga-lung-rgb-to-od", type_="npy")
+
+    expected_warning_regex = (
+        r"Input image must be RGB. NOTE: the image will be converted to RGB before"
+        r" OD conversion."
+    )
+
+    with pytest.warns(UserWarning, match=expected_warning_regex):
+        od_img = imf.rgb_to_od(img)
+
+    np.testing.assert_array_almost_equal(od_img, expected_value)
+
+
+def test_hed_to_rgb():
+    hed_arr = NPY.DIAGNOSTIC_SLIDE_THUMB_HED
+    expected_value = load_expectation(
+        "pil-images-rgb/diagnostic-slide-thumb-hed-to-rgb", type_="png"
+    )
+    # rountrip RGB<->HED is not possible, so
+    # "expectations/pil-images-rgb/diagnostic-slide-thumb-hed-to-rgb.png" won't be equal
+    # to "fixtures/pil-images-rgb/diagnostic-slide-thumb-rgb.png".
+    # See https://github.com/scikit-image/scikit-image/pull/5172
+
+    rgb_img = imf.hed_to_rgb(hed_arr)
+
+    np.testing.assert_array_almost_equal(rgb_img, expected_value)
+
+
+def test_lab_to_rgb_filter_with_rgb_image():
+    expected_value = load_expectation(
+        "pil-images-rgb/diagnostic-slide-thumb-lab-to-rgb", type_="png"
+    )
+
+    rgb_img = imf.lab_to_rgb(NPY.DIAGNOSTIC_SLIDE_THUMB_LAB)
+
+    np.testing.assert_array_almost_equal(np.array(rgb_img), np.array(expected_value))
+    assert np.unique(np.array(ImageChops.difference(rgb_img, expected_value)))[0] == 0
+
+
+def test_rgb_lab_roundtrip():
+    img = RGB.DIAGNOSTIC_SLIDE_THUMB_RGB
+
+    img_ = imf.lab_to_rgb(imf.rgb_to_lab(img))
+
+    np.testing.assert_array_almost_equal(np.array(img_), np.array(img))
+    assert np.unique(np.array(ImageChops.difference(img_, img)))[0] == 0
