@@ -40,13 +40,13 @@ class DescribeImageFilters:
         F_rgb_to_hed = function_mock(
             request, "histolab.filters.image_filters_functional.rgb_to_hed"
         )
-        F_rgb_to_hed.return_value = image
+        F_rgb_to_hed.return_value = np.array(image)
         rgb_to_hed = imf.RgbToHed()
 
         rgb_to_hed(image)
 
         F_rgb_to_hed.assert_called_once_with(image)
-        assert type(rgb_to_hed(image)) == PIL.Image.Image
+        assert type(rgb_to_hed(image)) == np.ndarray
 
     def it_calls_hematoxylin_channel_functional(self, request):
         image = PILIMG.RGBA_COLOR_500X500_155_249_240
@@ -401,13 +401,13 @@ class DescribeImageFilters:
         F_rgb_to_lab = function_mock(
             request, "histolab.filters.image_filters_functional.rgb_to_lab"
         )
-        F_rgb_to_lab.return_value = image
-        rgb_to_lab = imf.RgbToLab()
+        F_rgb_to_lab.return_value = np.array(image)
+        rgb_to_lab = imf.RgbToLab("A", "2")
 
         rgb_to_lab(image)
 
-        F_rgb_to_lab.assert_called_once_with(image)
-        assert type(rgb_to_lab(image)) == PIL.Image.Image
+        F_rgb_to_lab.assert_called_once_with(image, "A", "2")
+        assert type(rgb_to_lab(image)) == np.ndarray
 
     def it_calls_dab_channel_functional(self, request):
         image = PILIMG.RGBA_COLOR_500X500_155_249_240
@@ -421,3 +421,44 @@ class DescribeImageFilters:
 
         F_dab_channel.assert_called_once_with(image)
         assert type(dab_channel(image)) == PIL.Image.Image
+
+    def it_calls_rgb_to_od_functional(self, request):
+        image = np.array(PILIMG.RGBA_COLOR_500X500_155_249_240)
+        F_rgb_to_od = function_mock(
+            request, "histolab.filters.image_filters_functional.rgb_to_od"
+        )
+        F_rgb_to_od.return_value = image
+        rgb_to_od = imf.RgbToOd()
+
+        im_od = rgb_to_od(image)
+
+        F_rgb_to_od.assert_called_once_with(image)
+        assert isinstance(im_od, np.ndarray)
+
+    def it_calls_hed_to_rgb_functional(self, request):
+        image = PILIMG.RGBA_COLOR_500X500_155_249_240
+        np_image = np.array(image)
+        F_hed_to_rgb = function_mock(
+            request, "histolab.filters.image_filters_functional.hed_to_rgb"
+        )
+        F_hed_to_rgb.return_value = image
+        hed_to_rgb = imf.HedToRgb()
+
+        hed_to_rgb(np_image)
+
+        F_hed_to_rgb.assert_called_once_with(np_image)
+        assert type(hed_to_rgb(np_image)) == PIL.Image.Image
+
+    def it_calls_lab_to_rgb_functional(self, request):
+        image = PILIMG.RGBA_COLOR_500X500_155_249_240
+        image_arr = np.array(image)
+        F_lab_to_rgb = function_mock(
+            request, "histolab.filters.image_filters_functional.lab_to_rgb"
+        )
+        F_lab_to_rgb.return_value = image
+        lab_to_rgb = imf.LabToRgb("A", "2")
+
+        lab_to_rgb(image_arr)
+
+        F_lab_to_rgb.assert_called_once_with(image_arr, "A", "2")
+        assert type(lab_to_rgb(image_arr)) == PIL.Image.Image
