@@ -3,7 +3,6 @@
 import errno
 import math
 import os
-from collections import namedtuple
 from pathlib import Path
 
 import numpy as np
@@ -12,10 +11,9 @@ import PIL
 import pytest
 from PIL import ImageShow
 
-from histolab.exceptions import LevelError
+from histolab.exceptions import LevelError, SlidePropertyError
 from histolab.slide import Slide, SlideSet
-from histolab.types import CP, Region
-from histolab.util import regions_from_binary_mask
+from histolab.types import CP
 
 from ..unitutil import (
     ANY,
@@ -24,7 +22,6 @@ from ..unitutil import (
     call,
     class_mock,
     dict_list_eq,
-    function_mock,
     initializer_mock,
     instance_mock,
     is_win32,
@@ -237,38 +234,6 @@ class Describe_Slide:
 
         assert type(thumb) == PIL.Image.Image
 
-    def it_knows_regions_from_binary_mask(self, request):
-        binary_mask = np.array([[True, False], [True, True]])
-        label = function_mock(request, "histolab.util.label")
-        regionprops = function_mock(request, "histolab.util.regionprops")
-        RegionProps = namedtuple("RegionProps", ("area", "bbox", "centroid", "coords"))
-        regions_props = [
-            RegionProps(
-                3,
-                (0, 0, 2, 2),
-                (0.6666666666666666, 0.3333333333333333),
-                np.array([[0, 0], [1, 0], [1, 1]]),
-            )
-        ]
-        regionprops.return_value = regions_props
-        label(binary_mask).return_value = [[1, 0], [1, 1]]
-
-        regions_from_binary_mask_ = regions_from_binary_mask(binary_mask)
-
-        regionprops.assert_called_once_with(label(binary_mask))
-        assert type(regions_from_binary_mask_) == list
-        assert len(regions_from_binary_mask_) == 1
-        assert type(regions_from_binary_mask_[0]) == Region
-        assert regions_from_binary_mask_ == [
-            Region(
-                index=0,
-                area=regions_props[0].area,
-                bbox=regions_props[0].bbox,
-                center=regions_props[0].centroid,
-                coords=regions_props[0].coords,
-            )
-        ]
-
     @pytest.mark.skipif(
         not on_ci() or is_win32(), reason="Only run on CIs; hangs on Windows CIs"
     )
@@ -300,7 +265,7 @@ class Describe_Slide:
         assert level_dimensions == expected_value
 
     @pytest.mark.parametrize("level", (3, -3))
-    def but_it_raises_expection_when_level_does_not_exist(self, level, tmpdir):
+    def but_it_raises_exception_when_level_does_not_exist(self, level, tmpdir):
         slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
 
         with pytest.raises(LevelError) as err:
@@ -312,16 +277,120 @@ class Describe_Slide:
             == f"Level {level} not available. Number of available levels: 1"
         )
 
+    @pytest.mark.parametrize(
+        "properties",
+        (
+            (
+                {
+                    "openslide.objective-power": "20",
+                    "openslide.level[1].downsample": "4.003",
+                    "openslide.level[2].downsample": "16",
+                }
+            ),
+        ),
+    )
+    def it_knows_its_magnification(self, request, properties, tmpdir):
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
+        property_mock(request, Slide, "properties", return_value=properties)
+        property_mock(request, Slide, "levels", return_value=[0, 1, 2])
+
+        assert slide.level_magnification_factor(1) == "5.0X"
+        assert slide.level_magnification_factor(2) == "1.25X"
+
+    @pytest.mark.parametrize(
+        "properties, error",
+        (
+            (
+                {
+                    "openslide.objective-power": "20",
+                },
+                "Downsample factor for level 1 not available. Available slide "
+                "properties: ['openslide.objective-power']",
+            ),
+            (
+                {
+                    "openslide.level[1].downsample": "4.003",
+                    "openslide.level[2].downsample": "16",
+                },
+                "Native magnification not available. Available slide properties: "
+                "['openslide.level[1].downsample', 'openslide.level[2].downsample']",
+            ),
+        ),
+    )
+    def but_it_raises_an_exception_if_metadata_are_unavailable(
+        self, request, properties, error, tmpdir
+    ):
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
+        property_mock(request, Slide, "properties", return_value=properties)
+        property_mock(request, Slide, "levels", return_value=[0, 1, 2])
+        with pytest.raises(SlidePropertyError) as err:
+            slide.level_magnification_factor(1)
+
+        assert isinstance(err.value, SlidePropertyError)
+        assert str(err.value) == error
+
+    @pytest.mark.parametrize(
+        "properties",
+        (
+            (
+                {
+                    "openslide.objective-power": "20",
+                    "openslide.level[1].downsample": "4.003",
+                    "openslide.level[2].downsample": "16",
+                },
+            ),
+        ),
+    )
+    def and_it_raises_an_exception_if_level_in_incorrect(
+        self, request, properties, tmpdir
+    ):
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
+        property_mock(request, Slide, "properties", return_value=properties)
+        property_mock(request, Slide, "levels", return_value=[0, 1, 2])
+        with pytest.raises(LevelError) as err:
+            slide.level_magnification_factor(4)
+
+        assert isinstance(err.value, LevelError)
+        assert str(err.value) == "Level 4 not available. Number of available levels: 3"
+
+    @pytest.mark.parametrize("level", (1, -3))
+    def it_raises_an_exception_when_magnification_factor_is_unavailable(
+        self, level, tmpdir
+    ):
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
+
+        with pytest.raises(LevelError) as err:
+            slide.level_magnification_factor(level=level)
+
+        assert isinstance(err.value, LevelError)
+        assert (
+            str(err.value)
+            == f"Level {level} not available. Number of available levels: 1"
+        )
+
+    def it_raises_an_exception_when_native_magnification_in_unavailable(self, tmpdir):
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
+
+        with pytest.raises(SlidePropertyError) as err:
+            slide.level_magnification_factor()
+
+        assert isinstance(err.value, SlidePropertyError)
+        assert (
+            str(err.value)
+            == "Native magnification not available. Available slide properties: []"
+        )
+
     @pytest.mark.parametrize(
         "coords, expected_result",
         (
-            (CP(0, 128, 0, 128), True),
-            (CP(800000, 90000, 8000010, 90010), False),
-            (CP(800000, 90000, -1, 90010), False),
+            (CP(0, 40, 0, 40), True),  # point
+            (CP(0, 0, 48, 50), True),  # valid box
+            (CP(800000, 90000, 8000010, 90010), False),  # out of bounds box
+            (CP(800000, 90000, -1, 90010), False),  # negative coordinates
         ),
     )
     def it_knows_if_coords_are_valid(self, coords, expected_result, tmpdir):
-        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_49X51_155_0_0)
 
         _are_valid = slide._has_valid_coords(coords)
 
